from django.conf import settings
from django.contrib.messages.constants import DEFAULT_TAGS, INFO
from django.contrib.sites.models import Site
from django.db import models
from django.utils import timezone


# Create your models here.
class Tip(models.Model):
    name = models.CharField(max_length=128, null=False, blank=False)
    text = models.TextField()
    level = models.SmallIntegerField(choices=DEFAULT_TAGS.items(), default=INFO)

    run_start = models.DateTimeField(default=timezone.now)
    run_end = models.DateTimeField(null=True, blank=True)

    view = models.CharField(max_length=256, blank=True, null=True)
    sites = models.ManyToManyField(Site)

    seen_by = models.ManyToManyField(
        settings.AUTH_USER_MODEL, related_name="seen_tips", blank=True
    )

    def tags(self):
        return settings.MESSAGE_TAGS[self.level]

    def __str__(self):
        return self.name



# Generated by Django 2.0 on 2018-09-22 05:16

import django.utils.timezone
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ("sites", "0002_alter_domain_unique"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="Tip",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=128)),
                ("text", models.TextField()),
                (
                    "level",
                    models.SmallIntegerField(
                        choices=[
                            (10, "debug"),
                            (20, "info"),
                            (25, "success"),
                            (30, "warning"),
                            (40, "error"),
                        ],
                        default=20,
                    ),
                ),
                ("run_start", models.DateTimeField(default=django.utils.timezone.now)),
                ("run_end", models.DateTimeField(blank=True, null=True)),
                ("view", models.CharField(blank=True, max_length=256, null=True)),
                (
                    "seen_by",
                    models.ManyToManyField(
                        blank=True,
                        related_name="seen_tips",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                ("sites", models.ManyToManyField(to="sites.Site")),
            ],
        )
    ]



import datetime

from django.db.models import Q

from .models import Tip


def tips(request):
    """
    Adds a list of tips for the current request
    """
    if not request.user.is_authenticated:
        return {}

    # import pdb; pdb.set_trace()
    try:
        tips = Tip.objects.filter(run_start__lte=datetime.datetime.now())
        tips = tips.filter(
            Q(run_end__isnull=True) | Q(run_end__gte=datetime.datetime.now())
        )
        tips = tips.filter(
            Q(view="") | Q(view=request.resolver_match.url_name)
        ).exclude(seen_by=request.user)
        if len(tips) > 0:
            tips[0].seen_by.add(request.user)

            return {"tip": tips[0]}
    except:
        pass  # Don't show a TotD if anything went wrong
    return {}



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="django-totd", # Replace with your own username
    version="1.0.0",
    author="Michael Hall",
    author_email="mhall119@gmail.com",
    description="Django app for adding a Tip of the Day to any page",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/GetTogetherComm/totd",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    install_requires=[
        'Django>=2.0.12',
    ],
)


from django import forms
from django.contrib import admin
from django.urls.resolvers import get_resolver

from .models import Tip


def url_choices():
    choices = [("", "-- All Pages --")]
    for entry in get_resolver(None).url_patterns:
        if entry.pattern.name:
            choices.append((entry.pattern.name, entry.pattern.name))
    return choices


# Register your models here.
class TipForm(forms.ModelForm):
    class Meta:
        model = Tip
        fields = "__all__"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields["view"].widget = forms.Select(choices=url_choices())


class TipAdmin(admin.ModelAdmin):
    list_filter = ("level", "view")
    list_display = ("name", "level", "view", "run_start", "run_end", "seen_count")
    search_fields = ("name", "view")
    form = TipForm

    def seen_count(self, tip):
        return tip.seen_by.count()

    seen_count.short_description = "Seen by"


admin.site.register(Tip, TipAdmin)
